(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/forms'), require('angular-i18next')) :
    typeof define === 'function' && define.amd ? define('angular-validation-message', ['exports', '@angular/core', '@angular/forms', 'angular-i18next'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['angular-validation-message'] = {}, global.ng.core, global.ng.forms, global.angularI18next));
}(this, (function (exports, core, forms, angularI18next) { 'use strict';

    var ValidationMessage = /** @class */ (function () {
        function ValidationMessage(key, params) {
            if (key === void 0) { key = ''; }
            if (params === void 0) { params = null; }
            this.key = '';
            this.key = key;
            this.params = params;
        }
        return ValidationMessage;
    }());

    var ValidationMessageComponent = /** @class */ (function () {
        function ValidationMessageComponent(formControlName, formGroupName) {
            this.formControlName = formControlName;
            this.for = null;
            this.acd = formControlName || formGroupName;
            if (this.acd != null)
                this.for = this.acd.control;
        }
        Object.defineProperty(ValidationMessageComponent.prototype, "standalone", {
            get: function () {
                return this.acd == null;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(ValidationMessageComponent.prototype, "messages", {
            get: function () {
                return this.getValidationMessages();
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(ValidationMessageComponent.prototype, "firstMessage", {
            get: function () {
                return this.messages[0] ? this.messages[0] : new ValidationMessage();
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(ValidationMessageComponent.prototype, "controlPath", {
            get: function () {
                if (this.acd == null)
                    return '';
                return this.acd.path.join('.');
            },
            enumerable: false,
            configurable: true
        });
        ValidationMessageComponent.prototype.getValidationMessages = function () {
            var _this = this;
            if (this.for == null || this.for.errors == null)
                return [];
            var messages = [];
            var keys = Object.keys(this.for.errors);
            keys.forEach(function (k) {
                var errValue = _this.for.errors[k];
                var params = null;
                if (errValue instanceof Object)
                    params = errValue;
                else if (errValue !== true) {
                    params = {};
                    params[k] = errValue;
                }
                messages.push(new ValidationMessage(k, params));
            });
            return messages;
        };
        return ValidationMessageComponent;
    }());
    ValidationMessageComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'validation-message',
                    template: "\n      <div class=\"error-container\">{{ firstMessage.key + (firstMessage.params ? '(' + firstMessage.params + ')' : '') }}</div>\n      <i class=\"error-icon\"></i>\n    ",
                    encapsulation: core.ViewEncapsulation.None,
                    styles: ["\n      validation-message {\n        display: none;\n        width: 100%;\n      }\n      validation-message.standalone,\n      .ng-dirty.ng-invalid + validation-message {\n        display: block;\n      }\n    "]
                },] }
    ];
    ValidationMessageComponent.ctorParameters = function () { return [
        { type: forms.FormControlName, decorators: [{ type: core.Optional }] },
        { type: forms.FormGroupName, decorators: [{ type: core.Optional }] }
    ]; };
    ValidationMessageComponent.propDecorators = {
        standalone: [{ type: core.HostBinding, args: ['class.standalone',] }],
        for: [{ type: core.Input }]
    };

    var ValidationMessageDirective = /** @class */ (function () {
        function ValidationMessageDirective(viewContainer, componentFactoryResolver, vmComp) {
            this.viewContainer = viewContainer;
            this.componentFactoryResolver = componentFactoryResolver;
            this.vmComp = vmComp;
            this.validationMessageComponent = null;
        }
        ValidationMessageDirective.prototype.ngAfterViewInit = function () {
            var factory = this.componentFactoryResolver.resolveComponentFactory(this.vmComp);
            this.ngOnDestroy();
            this.validationMessageComponent = this.viewContainer.createComponent(factory, null, this.viewContainer.injector);
            this.validationMessageComponent.changeDetectorRef.detectChanges();
        };
        ValidationMessageDirective.prototype.ngOnDestroy = function () {
            if (this.validationMessageComponent) {
                this.validationMessageComponent.changeDetectorRef.detach();
            }
        };
        return ValidationMessageDirective;
    }());
    ValidationMessageDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: '[validationMessage]'
                },] }
    ];
    ValidationMessageDirective.ctorParameters = function () { return [
        { type: core.ViewContainerRef },
        { type: core.ComponentFactoryResolver },
        { type: core.Type, decorators: [{ type: core.Inject, args: [ValidationMessageComponent,] }] }
    ]; };

    var ValidationError = /** @class */ (function () {
        function ValidationError() {
        }
        Object.defineProperty(ValidationError.prototype, "error", {
            get: function () {
                return this.control.getError(this.errorKey);
            },
            enumerable: false,
            configurable: true
        });
        return ValidationError;
    }());

    var ValidationErrorBuilder = /** @class */ (function () {
        function ValidationErrorBuilder() {
        }
        ValidationErrorBuilder.prototype.build = function (form, prevKey) {
            var validationErrors = [];
            var formControls = form.controls;
            for (var key in formControls) {
                if (formControls.hasOwnProperty(key)) {
                    var fullKey = prevKey ? [prevKey, key].join('.') : key;
                    var ctrl = formControls[key];
                    if (ctrl instanceof forms.FormControl) {
                        if (!ctrl.valid) {
                            for (var erKey in ctrl.errors) {
                                if (ctrl.errors.hasOwnProperty(erKey)) {
                                    var valError = new ValidationError();
                                    valError.control = ctrl;
                                    valError.controlKey = fullKey;
                                    valError.errorKey = erKey;
                                    validationErrors.push(valError);
                                }
                            }
                        }
                    }
                    else {
                        var group = ctrl;
                        validationErrors = validationErrors.concat(this.build(group, fullKey));
                    }
                }
            }
            return validationErrors;
        };
        return ValidationErrorBuilder;
    }());

    var declarations$1 = [
        ValidationMessageDirective,
        ValidationMessageComponent,
    ];
    var ɵ0$1 = ValidationMessageComponent;
    var ValidationMessageModule = /** @class */ (function () {
        function ValidationMessageModule() {
        }
        return ValidationMessageModule;
    }());
    ValidationMessageModule.decorators = [
        { type: core.NgModule, args: [{
                    declarations: declarations$1,
                    exports: declarations$1,
                    entryComponents: [
                        ValidationMessageComponent
                    ],
                    providers: [
                        ValidationErrorBuilder,
                        {
                            provide: ValidationMessageComponent,
                            useValue: ɵ0$1
                        }
                    ]
                },] }
    ];

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
    /* global Reflect, Promise */
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b)
                if (Object.prototype.hasOwnProperty.call(b, p))
                    d[p] = b[p]; };
        return extendStatics(d, b);
    };
    function __extends(d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }
    var __assign = function () {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s)
                    if (Object.prototype.hasOwnProperty.call(s, p))
                        t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };
    function __rest(s, e) {
        var t = {};
        for (var p in s)
            if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
                t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                    t[p[i]] = s[p[i]];
            }
        return t;
    }
    function __decorate(decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
            r = Reflect.decorate(decorators, target, key, desc);
        else
            for (var i = decorators.length - 1; i >= 0; i--)
                if (d = decorators[i])
                    r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    }
    function __param(paramIndex, decorator) {
        return function (target, key) { decorator(target, key, paramIndex); };
    }
    function __metadata(metadataKey, metadataValue) {
        if (typeof Reflect === "object" && typeof Reflect.metadata === "function")
            return Reflect.metadata(metadataKey, metadataValue);
    }
    function __awaiter(thisArg, _arguments, P, generator) {
        function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try {
                step(generator.next(value));
            }
            catch (e) {
                reject(e);
            } }
            function rejected(value) { try {
                step(generator["throw"](value));
            }
            catch (e) {
                reject(e);
            } }
            function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }
    function __generator(thisArg, body) {
        var _ = { label: 0, sent: function () { if (t[0] & 1)
                throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function () { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f)
                throw new TypeError("Generator is already executing.");
            while (_)
                try {
                    if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done)
                        return t;
                    if (y = 0, t)
                        op = [op[0] & 2, t.value];
                    switch (op[0]) {
                        case 0:
                        case 1:
                            t = op;
                            break;
                        case 4:
                            _.label++;
                            return { value: op[1], done: false };
                        case 5:
                            _.label++;
                            y = op[1];
                            op = [0];
                            continue;
                        case 7:
                            op = _.ops.pop();
                            _.trys.pop();
                            continue;
                        default:
                            if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) {
                                _ = 0;
                                continue;
                            }
                            if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) {
                                _.label = op[1];
                                break;
                            }
                            if (op[0] === 6 && _.label < t[1]) {
                                _.label = t[1];
                                t = op;
                                break;
                            }
                            if (t && _.label < t[2]) {
                                _.label = t[2];
                                _.ops.push(op);
                                break;
                            }
                            if (t[2])
                                _.ops.pop();
                            _.trys.pop();
                            continue;
                    }
                    op = body.call(thisArg, _);
                }
                catch (e) {
                    op = [6, e];
                    y = 0;
                }
                finally {
                    f = t = 0;
                }
            if (op[0] & 5)
                throw op[1];
            return { value: op[0] ? op[1] : void 0, done: true };
        }
    }
    var __createBinding = Object.create ? (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        Object.defineProperty(o, k2, { enumerable: true, get: function () { return m[k]; } });
    }) : (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        o[k2] = m[k];
    });
    function __exportStar(m, o) {
        for (var p in m)
            if (p !== "default" && !Object.prototype.hasOwnProperty.call(o, p))
                __createBinding(o, m, p);
    }
    function __values(o) {
        var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
        if (m)
            return m.call(o);
        if (o && typeof o.length === "number")
            return {
                next: function () {
                    if (o && i >= o.length)
                        o = void 0;
                    return { value: o && o[i++], done: !o };
                }
            };
        throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }
    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m)
            return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done)
                ar.push(r.value);
        }
        catch (error) {
            e = { error: error };
        }
        finally {
            try {
                if (r && !r.done && (m = i["return"]))
                    m.call(i);
            }
            finally {
                if (e)
                    throw e.error;
            }
        }
        return ar;
    }
    /** @deprecated */
    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }
    /** @deprecated */
    function __spreadArrays() {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++)
            s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    }
    function __spreadArray(to, from, pack) {
        if (pack || arguments.length === 2)
            for (var i = 0, l = from.length, ar; i < l; i++) {
                if (ar || !(i in from)) {
                    if (!ar)
                        ar = Array.prototype.slice.call(from, 0, i);
                    ar[i] = from[i];
                }
            }
        return to.concat(ar || from);
    }
    function __await(v) {
        return this instanceof __await ? (this.v = v, this) : new __await(v);
    }
    function __asyncGenerator(thisArg, _arguments, generator) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var g = generator.apply(thisArg, _arguments || []), i, q = [];
        return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
        function verb(n) { if (g[n])
            i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
        function resume(n, v) { try {
            step(g[n](v));
        }
        catch (e) {
            settle(q[0][3], e);
        } }
        function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
        function fulfill(value) { resume("next", value); }
        function reject(value) { resume("throw", value); }
        function settle(f, v) { if (f(v), q.shift(), q.length)
            resume(q[0][0], q[0][1]); }
    }
    function __asyncDelegator(o) {
        var i, p;
        return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
        function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
    }
    function __asyncValues(o) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var m = o[Symbol.asyncIterator], i;
        return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
        function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
        function settle(resolve, reject, d, v) { Promise.resolve(v).then(function (v) { resolve({ value: v, done: d }); }, reject); }
    }
    function __makeTemplateObject(cooked, raw) {
        if (Object.defineProperty) {
            Object.defineProperty(cooked, "raw", { value: raw });
        }
        else {
            cooked.raw = raw;
        }
        return cooked;
    }
    ;
    var __setModuleDefault = Object.create ? (function (o, v) {
        Object.defineProperty(o, "default", { enumerable: true, value: v });
    }) : function (o, v) {
        o["default"] = v;
    };
    function __importStar(mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k))
                    __createBinding(result, mod, k);
        __setModuleDefault(result, mod);
        return result;
    }
    function __importDefault(mod) {
        return (mod && mod.__esModule) ? mod : { default: mod };
    }
    function __classPrivateFieldGet(receiver, state, kind, f) {
        if (kind === "a" && !f)
            throw new TypeError("Private accessor was defined without a getter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver))
            throw new TypeError("Cannot read private member from an object whose class did not declare it");
        return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
    }
    function __classPrivateFieldSet(receiver, state, value, kind, f) {
        if (kind === "m")
            throw new TypeError("Private method is not writable");
        if (kind === "a" && !f)
            throw new TypeError("Private accessor was defined without a setter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver))
            throw new TypeError("Cannot write private member to an object whose class did not declare it");
        return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
    }

    var I18NextValidationMessageComponent = /** @class */ (function (_super) {
        __extends(I18NextValidationMessageComponent, _super);
        function I18NextValidationMessageComponent(i18nextNamespace, formControlName, formGroupName) {
            var _this = _super.call(this, formControlName, formGroupName) || this;
            _this.i18nextNamespace = i18nextNamespace;
            _this.formControlName = formControlName;
            _this.formGroupName = formGroupName;
            _this.VALIDATION = 'validation';
            return _this;
        }
        Object.defineProperty(I18NextValidationMessageComponent.prototype, "i18nextKey", {
            get: function () {
                if (!this.firstMessage.key)
                    return '';
                var specificKey = [this.VALIDATION, ['control_specific', this.controlPath, this.firstMessage.key].join('.')].join(':');
                var commonKey = [this.VALIDATION, this.firstMessage.key].join(':');
                var i18nextKeys = [];
                if (this.i18nextNamespace && this.i18nextNamespace !== this.VALIDATION) {
                    i18nextKeys.push([this.i18nextNamespace, specificKey].join('.'));
                    i18nextKeys.push([this.i18nextNamespace, commonKey].join('.'));
                }
                i18nextKeys.push(specificKey);
                i18nextKeys.push(commonKey);
                return i18nextKeys;
            },
            enumerable: false,
            configurable: true
        });
        return I18NextValidationMessageComponent;
    }(ValidationMessageComponent));
    I18NextValidationMessageComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'i18next-validation-message',
                    template: "\n      <div class=\"error-container\">{{ i18nextKey | i18nextCap: firstMessage.params }}</div>\n      <i class=\"error-icon\"></i>\n    ",
                    encapsulation: core.ViewEncapsulation.None,
                    styles: ["\n      i18next-validation-message {\n        display: none;\n        width: 100%;\n        position: relative;\n      }\n      i18next-validation-message.standalone,\n      .ng-dirty.ng-invalid + i18next-validation-message {\n        display: block;\n      }\n    "]
                },] }
    ];
    I18NextValidationMessageComponent.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: core.Inject, args: [angularI18next.I18NEXT_NAMESPACE,] }] },
        { type: forms.FormControlName, decorators: [{ type: core.Optional }] },
        { type: forms.FormGroupName, decorators: [{ type: core.Optional }] }
    ]; };

    var declarations = [
        I18NextValidationMessageComponent,
    ];
    var ɵ0 = I18NextValidationMessageComponent;
    var providers = [
        {
            provide: ValidationMessageComponent,
            useValue: ɵ0
        }
    ];
    var I18NextValidationMessageModule = /** @class */ (function () {
        function I18NextValidationMessageModule() {
        }
        return I18NextValidationMessageModule;
    }());
    I18NextValidationMessageModule.decorators = [
        { type: core.NgModule, args: [{
                    declarations: declarations,
                    entryComponents: declarations,
                    exports: [
                        declarations,
                        ValidationMessageModule
                    ],
                    imports: [
                        angularI18next.I18NextModule,
                        ValidationMessageModule
                    ],
                    providers: providers
                },] }
    ];

    /**
     * Generated bundle index. Do not edit.
     */

    exports.I18NextValidationMessageModule = I18NextValidationMessageModule;
    exports.ValidationError = ValidationError;
    exports.ValidationErrorBuilder = ValidationErrorBuilder;
    exports.ValidationMessage = ValidationMessage;
    exports.ValidationMessageComponent = ValidationMessageComponent;
    exports.ValidationMessageDirective = ValidationMessageDirective;
    exports.ValidationMessageModule = ValidationMessageModule;
    exports.declarations = declarations$1;
    exports.providers = providers;
    exports.ɵa = I18NextValidationMessageComponent;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=angular-validation-message.umd.js.map
