import { Component, ViewEncapsulation, Optional, HostBinding, Input, Directive, ViewContainerRef, ComponentFactoryResolver, Type, Inject, NgModule } from '@angular/core';
import { FormControlName, FormGroupName, FormControl } from '@angular/forms';
import { I18NEXT_NAMESPACE, I18NextModule } from 'angular-i18next';

class ValidationMessage {
    constructor(key = '', params = null) {
        this.key = '';
        this.key = key;
        this.params = params;
    }
}

class ValidationMessageComponent {
    constructor(formControlName, formGroupName) {
        this.formControlName = formControlName;
        this.for = null;
        this.acd = formControlName || formGroupName;
        if (this.acd != null)
            this.for = this.acd.control;
    }
    get standalone() {
        return this.acd == null;
    }
    get messages() {
        return this.getValidationMessages();
    }
    get firstMessage() {
        return this.messages[0] ? this.messages[0] : new ValidationMessage();
    }
    get controlPath() {
        if (this.acd == null)
            return '';
        return this.acd.path.join('.');
    }
    getValidationMessages() {
        if (this.for == null || this.for.errors == null)
            return [];
        let messages = [];
        let keys = Object.keys(this.for.errors);
        keys.forEach(k => {
            let errValue = this.for.errors[k];
            let params = null;
            if (errValue instanceof Object)
                params = errValue;
            else if (errValue !== true) {
                params = {};
                params[k] = errValue;
            }
            messages.push(new ValidationMessage(k, params));
        });
        return messages;
    }
}
ValidationMessageComponent.decorators = [
    { type: Component, args: [{
                selector: 'validation-message',
                template: `
      <div class="error-container">{{ firstMessage.key + (firstMessage.params ? '(' + firstMessage.params + ')' : '') }}</div>
      <i class="error-icon"></i>
    `,
                encapsulation: ViewEncapsulation.None,
                styles: [`
      validation-message {
        display: none;
        width: 100%;
      }
      validation-message.standalone,
      .ng-dirty.ng-invalid + validation-message {
        display: block;
      }
    `]
            },] }
];
ValidationMessageComponent.ctorParameters = () => [
    { type: FormControlName, decorators: [{ type: Optional }] },
    { type: FormGroupName, decorators: [{ type: Optional }] }
];
ValidationMessageComponent.propDecorators = {
    standalone: [{ type: HostBinding, args: ['class.standalone',] }],
    for: [{ type: Input }]
};

class ValidationMessageDirective {
    constructor(viewContainer, componentFactoryResolver, vmComp) {
        this.viewContainer = viewContainer;
        this.componentFactoryResolver = componentFactoryResolver;
        this.vmComp = vmComp;
        this.validationMessageComponent = null;
    }
    ngAfterViewInit() {
        let factory = this.componentFactoryResolver.resolveComponentFactory(this.vmComp);
        this.ngOnDestroy();
        this.validationMessageComponent = this.viewContainer.createComponent(factory, null, this.viewContainer.injector);
        this.validationMessageComponent.changeDetectorRef.detectChanges();
    }
    ngOnDestroy() {
        if (this.validationMessageComponent) {
            this.validationMessageComponent.changeDetectorRef.detach();
        }
    }
}
ValidationMessageDirective.decorators = [
    { type: Directive, args: [{
                selector: '[validationMessage]'
            },] }
];
ValidationMessageDirective.ctorParameters = () => [
    { type: ViewContainerRef },
    { type: ComponentFactoryResolver },
    { type: Type, decorators: [{ type: Inject, args: [ValidationMessageComponent,] }] }
];

class ValidationError {
    get error() {
        return this.control.getError(this.errorKey);
    }
}

class ValidationErrorBuilder {
    build(form, prevKey) {
        let validationErrors = [];
        let formControls = form.controls;
        for (const key in formControls) {
            if (formControls.hasOwnProperty(key)) {
                let fullKey = prevKey ? [prevKey, key].join('.') : key;
                let ctrl = formControls[key];
                if (ctrl instanceof FormControl) {
                    if (!ctrl.valid) {
                        for (const erKey in ctrl.errors) {
                            if (ctrl.errors.hasOwnProperty(erKey)) {
                                let valError = new ValidationError();
                                valError.control = ctrl;
                                valError.controlKey = fullKey;
                                valError.errorKey = erKey;
                                validationErrors.push(valError);
                            }
                        }
                    }
                }
                else {
                    let group = ctrl;
                    validationErrors = validationErrors.concat(this.build(group, fullKey));
                }
            }
        }
        return validationErrors;
    }
}

const declarations$1 = [
    ValidationMessageDirective,
    ValidationMessageComponent,
];
const ɵ0$1 = ValidationMessageComponent;
class ValidationMessageModule {
}
ValidationMessageModule.decorators = [
    { type: NgModule, args: [{
                declarations: declarations$1,
                exports: declarations$1,
                entryComponents: [
                    ValidationMessageComponent
                ],
                providers: [
                    ValidationErrorBuilder,
                    {
                        provide: ValidationMessageComponent,
                        useValue: ɵ0$1
                    }
                ]
            },] }
];

class I18NextValidationMessageComponent extends ValidationMessageComponent {
    constructor(i18nextNamespace, formControlName, formGroupName) {
        super(formControlName, formGroupName);
        this.i18nextNamespace = i18nextNamespace;
        this.formControlName = formControlName;
        this.formGroupName = formGroupName;
        this.VALIDATION = 'validation';
    }
    get i18nextKey() {
        if (!this.firstMessage.key)
            return '';
        let specificKey = [this.VALIDATION, ['control_specific', this.controlPath, this.firstMessage.key].join('.')].join(':');
        let commonKey = [this.VALIDATION, this.firstMessage.key].join(':');
        let i18nextKeys = [];
        if (this.i18nextNamespace && this.i18nextNamespace !== this.VALIDATION) {
            i18nextKeys.push([this.i18nextNamespace, specificKey].join('.'));
            i18nextKeys.push([this.i18nextNamespace, commonKey].join('.'));
        }
        i18nextKeys.push(specificKey);
        i18nextKeys.push(commonKey);
        return i18nextKeys;
    }
}
I18NextValidationMessageComponent.decorators = [
    { type: Component, args: [{
                selector: 'i18next-validation-message',
                template: `
      <div class="error-container">{{ i18nextKey | i18nextCap: firstMessage.params }}</div>
      <i class="error-icon"></i>
    `,
                encapsulation: ViewEncapsulation.None,
                styles: [`
      i18next-validation-message {
        display: none;
        width: 100%;
        position: relative;
      }
      i18next-validation-message.standalone,
      .ng-dirty.ng-invalid + i18next-validation-message {
        display: block;
      }
    `]
            },] }
];
I18NextValidationMessageComponent.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [I18NEXT_NAMESPACE,] }] },
    { type: FormControlName, decorators: [{ type: Optional }] },
    { type: FormGroupName, decorators: [{ type: Optional }] }
];

const declarations = [
    I18NextValidationMessageComponent,
];
const ɵ0 = I18NextValidationMessageComponent;
const providers = [
    {
        provide: ValidationMessageComponent,
        useValue: ɵ0
    }
];
class I18NextValidationMessageModule {
}
I18NextValidationMessageModule.decorators = [
    { type: NgModule, args: [{
                declarations: declarations,
                entryComponents: declarations,
                exports: [
                    declarations,
                    ValidationMessageModule
                ],
                imports: [
                    I18NextModule,
                    ValidationMessageModule
                ],
                providers: providers
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { I18NextValidationMessageModule, ValidationError, ValidationErrorBuilder, ValidationMessage, ValidationMessageComponent, ValidationMessageDirective, ValidationMessageModule, declarations$1 as declarations, providers, I18NextValidationMessageComponent as ɵa };
//# sourceMappingURL=angular-validation-message.js.map
